@@ -45,6 +45,8 @@ AUDIO_PLAYLIST_CONTAINER_ID = 104
 AUDIO_GENRE_CONTAINER_ID = 105
 AUDIO_TAG_CONTAINER_ID = 106
 
+VIDEO_CONTAINER_ID = 200
+
 
 from urlparse import urlsplit
 
@@ -69,26 +71,30 @@ class Container(BackendItem):
 
     get_path = None
 
-    def __init__(self, id, parent_id, name, store=None, children_callback=None, container_class=DIDLLite.Container):
+    def __init__(self, id, parent_id, name, store=None, children_callback=None, container_class=DIDLLite.Container,play_container=False):
         self.id = id
         self.parent_id = parent_id
         self.name = name
         self.mimetype = 'directory'
-        self.item = container_class(id, parent_id,self.name)
+        self.container_class = container_class
         self.update_id = 0
         if children_callback != None:
             self.children = children_callback
         else:
             self.children = []
-        self.item.childCount = None #self.get_child_count()
+        self.childCount = None
 
-        if store!=None:
-            self.get_url = lambda: store.urlbase + str(self.id)
+        self.store = store
+        self.play_container = play_container
+
+        if self.store!=None:
+            self.get_url = lambda: self.store.urlbase + str(self.id)
 
     def add_child(self, child):
         self.children.append(child)
-        if self.item.childCount != None:
-            self.item.childCount += 1
+        if self.childCount == None:
+            self.childCount = 0
+        self.childCount += 1
 
     def get_children(self,start=0,end=0):
         self.info("container.get_children %r %r", start, end)
@@ -106,16 +112,32 @@ class Container(BackendItem):
             return children[start:end]
 
     def get_child_count(self):
-        if self.item.childCount != None:
-            return self.item.childCount
-
-        if callable(self.children):
-            return len(self.children())
-        else:
-            return len(self.children)
+        if self.childCount == None:
+            if callable(self.children):
+                self.childCount = len(self.children())
+            else:
+                self.childCount = len(self.children)
+        return self.childCount
 
     def get_item(self):
-        return self.item
+        item = self.container_class(self.id, self.parent_id,self.name)
+        item.childCount = self.get_child_count()
+        #if self.store and self.play_container == True:
+        #    if item.childCount > 0:
+        #        d = defer.maybeDeferred(self.get_children, 0, 1)
+
+        #        def process_result(r,item):
+        #            res = DIDLLite.PlayContainerResource(self.store.server.uuid,cid=self.get_id(),fid=r[0].get_id())
+        #            item.res.append(res)
+        #            return item
+
+        #        def got_error(f,item):
+        #            return item
+
+        #        d.addCallback(process_result,item)
+        #        d.addErrback(got_error,item)
+        #        return d
+        return item
 
     def get_name(self):
         return self.name
@@ -151,7 +173,7 @@ class Playlist(BackendItem):
     def get_item(self, parent_id = AUDIO_PLAYLIST_CONTAINER_ID):
         item = DIDLLite.PlaylistItem(self.id, parent_id, self.title)
         item.childCount = self.get_child_count()
-        item.artist = self.artist
+        #item.artist = self.artist
         item.albumArtURI = self.cover
         return item
 
@@ -194,6 +216,20 @@ class Album(BackendItem):
         item.childCount = self.get_child_count()
         item.artist = self.artist
         item.albumArtURI = self.cover
+        #if item.childCount > 0:
+        #    d = defer.maybeDeferred(self.get_children, 0, 1)
+
+        #    def process_result(r,item):
+        #        res = DIDLLite.PlayContainerResource(self.store.server.uuid,cid=self.get_id(),fid=r[0].get_id())
+        #        item.res.append(res)
+        #        return item
+
+        #    def got_error(f,item):
+        #        return item
+
+        #    d.addCallback(process_result,item)
+        #    d.addErrback(got_error,item)
+        #    return d
         return item
 
     def get_id(self):
@@ -449,6 +485,88 @@ class Track(BackendItem):
         return None
 
 
+class Video(BackendItem):
+
+    logCategory = 'ampache_store'
+
+    def __init__(self,store,element):
+        self.store = store
+        self.id = 'video.%d' % int(element.get('id'))
+
+        self.url = element.find('url').text
+
+        try:
+            seconds = int(element.find('time').text)
+            hours = seconds / 3600
+            seconds = seconds - hours * 3600
+            minutes = seconds / 60
+            seconds = seconds - minutes * 60
+            self.duration = ("%d:%02d:%02d") % (hours, minutes, seconds)
+        except:
+            self.duration = 0
+
+        self.cover = None
+
+
+        self.title = element.find('title').text
+
+        self.mimetype = None
+        try:
+            self.mimetype = element.find('mime').text
+        except:
+            self.mimetype,_ = mimetypes.guess_type(self.url, strict=False)
+        if self.mimetype == None:
+            self.mimetype = "video/avi"
+        try:
+            self.size = int(element.find('size').text)
+        except:
+            self.size = 0
+
+        if self.store.proxy == True:
+            self.location = ProxySong(self.url)
+
+    def get_children(self, start=0, request_count=0):
+        return []
+
+    def get_child_count(self):
+        return 0
+
+    def get_item(self, parent_id=VIDEO_CONTAINER_ID):
+
+        self.debug("video get_item %r @ %r" %(self.id,parent_id))
+
+        # create item
+        item = DIDLLite.VideoItem(self.id,parent_id)
+        item.title = self.title
+
+        item.albumArtURI = self.cover
+
+        # add http resource
+        res = DIDLLite.Resource(self.get_url(), 'http-get:*:%s:*' % self.mimetype)
+        if self.size > 0:
+            res.size = self.size
+        if self.duration > 0:
+            res.duration = str(self.duration)
+        item.res.append(res)
+
+        return item
+
+    def get_id(self):
+        return self.id
+
+    def get_name(self):
+        return self.title
+
+    def get_url(self):
+        if self.store.proxy == True:
+            return self.store.urlbase + str(self.id)
+        else:
+            return self.url
+
+    def get_path(self):
+        return None
+
+
 class AmpacheStore(BackendStore):
 
     """ this is a backend to the Ampache Media DB
@@ -459,9 +577,10 @@ class AmpacheStore(BackendStore):
     logCategory = 'ampache_store'
 
     def __init__(self, server, **kwargs):
+        BackendStore.__init__(self,server,**kwargs)
         self.config = kwargs
         self.name = kwargs.get('name','Ampache')
-        self.key = kwargs.get('key','')
+        self.key = kwargs.get('password',kwargs.get('key',''))
         self.user = kwargs.get('user',None)
         self.url = kwargs.get('url','http://localhost/ampache/server/xml.server.php')
 
@@ -470,12 +589,6 @@ class AmpacheStore(BackendStore):
         else:
             self.proxy = False
 
-        self.urlbase = kwargs.get('urlbase','')
-        if self.urlbase[len(self.urlbase)-1] != '/':
-            self.urlbase += '/'
-
-
-        self.server = server
         self.update_id = 0
         self.token = None
 
@@ -509,7 +622,7 @@ class AmpacheStore(BackendStore):
         except ValueError:
             try:
                 type,id = id.split('.')
-                if type in ['song','artist','album','playlist','genre','tag']:
+                if type in ['song','artist','album','playlist','genre','tag','video']:
                     item = self.ampache_query(type, filter=str(id))
             except ValueError:
                 return None
@@ -544,8 +657,13 @@ class AmpacheStore(BackendStore):
                     self.tags = int(response.find('tags').text)
                 except:
                     self.tags = 0
+                try:
+                    self.videos = int(response.find('videos').text)
+                except:
+                    self.videos = 0
                 self.info('ampache returned auth token %r', self.token)
-                self.info('Songs: %d, Artists: %d, Albums: %d, Playlists %d, Genres %d, Tags %d' % (self.songs, self.artists,self.albums,self.playlists,self.genres,self.tags))
+                self.info('Songs: %d, Artists: %d, Albums: %d, Playlists %d, Genres %d, Tags %d, Videos %d' %
+                          (self.songs, self.artists,self.albums,self.playlists,self.genres,self.tags,self.videos))
 
                 if renegotiate == False:
                     self.containers = {}
@@ -557,6 +675,7 @@ class AmpacheStore(BackendStore):
                                              '6': lambda : self.get_by_id(AUDIO_ARTIST_CONTAINER_ID),    # all artists
                                              '7': lambda : self.get_by_id(AUDIO_ALBUM_CONTAINER_ID),     # all albums
                                              '13': lambda : self.get_by_id(AUDIO_PLAYLIST_CONTAINER_ID), # all playlists
+                                             '8': lambda : self.get_by_id(VIDEO_CONTAINER_ID),          # all videos
                                             })
 
                     louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self)
@@ -592,6 +711,7 @@ class AmpacheStore(BackendStore):
 
     def got_response(self, response, query_item, request):
         self.info("got a response for %r", query_item)
+        self.debug(response)
         response = utils.parse_xml(response, encoding='utf-8')
         items = []
         try:
@@ -616,7 +736,7 @@ class AmpacheStore(BackendStore):
                 return d
             raise ValueError, error.text
         except AttributeError:
-            if query_item in ('song','artist','album','playlist','genre'):
+            if query_item in ('song','artist','album','playlist','genre','tag','video'):
                 q = response.find(query_item)
                 if q == None:
                     return None
@@ -633,8 +753,10 @@ class AmpacheStore(BackendStore):
                         return Genre(self,q)
                     if q.tag in ['tag']:
                         return Tag(self,q)
+                    if q.tag in ['video']:
+                        return Video(self,q)
             else:
-                if query_item in ('songs','artists','albums','playlists','genres','tag'):
+                if query_item in ('songs','artists','albums','playlists','genres','tags','videos'):
                     query_item = query_item[:-1]
                 if query_item in ('playlist_songs','album_songs','genre_songs','tag_songs'):
                     query_item = 'song'
@@ -653,6 +775,8 @@ class AmpacheStore(BackendStore):
                         items.append(Genre(self,q))
                     if query_item in ('tag',):
                         items.append(Tag(self,q))
+                    if query_item in ('video',):
+                        items.append(Video(self,q))
         return items
 
     def ampache_query(self, item, start=0, request_count=0, filter=None):
@@ -685,55 +809,71 @@ class AmpacheStore(BackendStore):
     def ampache_query_tags(self, start=0, request_count=0):
         return self.ampache_query('tags', start, request_count)
 
+    def ampache_query_videos(self, start=0, request_count=0):
+        return self.ampache_query('videos', start, request_count)
+
+
     def upnp_init(self):
         if self.server:
             self.server.connection_manager_server.set_variable(0, 'SourceProtocolInfo',
                             ['http-get:*:audio/mpeg:*',
-                             'http-get:*:application/ogg:*',])
+                             'http-get:*:application/ogg:*',
+                             'http-get:*:video/mp4:*',
+                             'http-get:*:video/x-msvideo:*',
+                             'http-get:*:video/avi:*',
+                             'http-get:*:video/quicktime:*',])
 
         self.containers[AUDIO_ALL_CONTAINER_ID] = \
-                Container( AUDIO_ALL_CONTAINER_ID,ROOT_CONTAINER_ID, 'All tracks',
+                Container(AUDIO_ALL_CONTAINER_ID,ROOT_CONTAINER_ID, 'All tracks',
                           store=self,
-                          children_callback=self.ampache_query_songs)
-        self.containers[AUDIO_ALL_CONTAINER_ID].item.childCount = self.songs
+                          children_callback=self.ampache_query_songs,
+                          play_container=True)
+        self.containers[AUDIO_ALL_CONTAINER_ID].childCount = self.songs
         self.containers[ROOT_CONTAINER_ID].add_child(self.containers[AUDIO_ALL_CONTAINER_ID])
 
         self.containers[AUDIO_ALBUM_CONTAINER_ID] = \
-                Container( AUDIO_ALBUM_CONTAINER_ID,ROOT_CONTAINER_ID, 'Albums',
+                Container(AUDIO_ALBUM_CONTAINER_ID,ROOT_CONTAINER_ID, 'Albums',
                           store=self,
                           children_callback=self.ampache_query_albums)
-        self.containers[AUDIO_ALBUM_CONTAINER_ID].item.childCount = self.albums
+        self.containers[AUDIO_ALBUM_CONTAINER_ID].childCount = self.albums
         self.containers[ROOT_CONTAINER_ID].add_child(self.containers[AUDIO_ALBUM_CONTAINER_ID])
 
         self.containers[AUDIO_ARTIST_CONTAINER_ID] = \
                 Container( AUDIO_ARTIST_CONTAINER_ID,ROOT_CONTAINER_ID, 'Artists',
                           store=self,
                           children_callback=self.ampache_query_artists)
-        self.containers[AUDIO_ARTIST_CONTAINER_ID].item.childCount = self.artists
+        self.containers[AUDIO_ARTIST_CONTAINER_ID].childCount = self.artists
         self.containers[ROOT_CONTAINER_ID].add_child(self.containers[AUDIO_ARTIST_CONTAINER_ID])
 
         self.containers[AUDIO_PLAYLIST_CONTAINER_ID] = \
-                Container( AUDIO_PLAYLIST_CONTAINER_ID,ROOT_CONTAINER_ID, 'Playlists',
+                Container(AUDIO_PLAYLIST_CONTAINER_ID,ROOT_CONTAINER_ID, 'Playlists',
                           store=self,
                           children_callback=self.ampache_query_playlists,
                           container_class=DIDLLite.PlaylistContainer)
-        self.containers[AUDIO_PLAYLIST_CONTAINER_ID].item.childCount = self.playlists
+        self.containers[AUDIO_PLAYLIST_CONTAINER_ID].childCount = self.playlists
         self.containers[ROOT_CONTAINER_ID].add_child(self.containers[AUDIO_PLAYLIST_CONTAINER_ID])
 
         self.containers[AUDIO_GENRE_CONTAINER_ID] = \
-                Container( AUDIO_GENRE_CONTAINER_ID,ROOT_CONTAINER_ID, 'Genres',
+                Container(AUDIO_GENRE_CONTAINER_ID,ROOT_CONTAINER_ID, 'Genres',
                           store=self,
                           children_callback=self.ampache_query_genres)
-        self.containers[AUDIO_GENRE_CONTAINER_ID].item.childCount = self.genres
+        self.containers[AUDIO_GENRE_CONTAINER_ID].childCount = self.genres
         self.containers[ROOT_CONTAINER_ID].add_child(self.containers[AUDIO_GENRE_CONTAINER_ID])
 
         self.containers[AUDIO_TAG_CONTAINER_ID] = \
-                Container( AUDIO_TAG_CONTAINER_ID,ROOT_CONTAINER_ID, 'Tags',
+                Container(AUDIO_TAG_CONTAINER_ID,ROOT_CONTAINER_ID, 'Tags',
                           store=self,
                           children_callback=self.ampache_query_tags)
-        self.containers[AUDIO_TAG_CONTAINER_ID].item.childCount = self.tags
+        self.containers[AUDIO_TAG_CONTAINER_ID].childCount = self.tags
         self.containers[ROOT_CONTAINER_ID].add_child(self.containers[AUDIO_TAG_CONTAINER_ID])
 
+        self.containers[VIDEO_CONTAINER_ID] = \
+                Container(VIDEO_CONTAINER_ID,ROOT_CONTAINER_ID, 'Videos',
+                          store=self,
+                          children_callback=self.ampache_query_videos)
+        self.containers[VIDEO_CONTAINER_ID].childCount = self.videos
+        self.containers[ROOT_CONTAINER_ID].add_child(self.containers[VIDEO_CONTAINER_ID])
+
     def upnp_XBrowse(self, *args, **kwargs):
         try:
             ObjectID = kwargs['ObjectID']
