@@ -11,28 +11,44 @@ This is a Media Backend that allows you to access the Trailers from Apple.com
 
 from coherence.backend import BackendItem, BackendStore
 from coherence.upnp.core import DIDLLite
+from coherence.upnp.core.utils import ReverseProxyUriResource
 from twisted.web import client
 from twisted.internet import task, reactor
 
 from coherence.extern.et import parse_xml
-import coherence.extern.louie as louie
-
 
 XML_URL = "http://www.apple.com/trailers/home/xml/current.xml"
 
 ROOT_ID = 0
 
+class AppleTrailerProxy(ReverseProxyUriResource):
+
+    def __init__(self, uri):
+        ReverseProxyUriResource.__init__(self, uri)
+
+    def render(self, request):
+        request.received_headers['user-agent'] = 'QuickTime/7.6.2 (qtver=7.6.2;os=Windows NT 5.1Service Pack 3)'
+        return ReverseProxyUriResource.render(self, request)
+
 
 class Trailer(BackendItem):
 
-    def __init__(self, parent_id, id=None, name=None, cover=None,
-            location=None):
+    def __init__(self, parent_id, urlbase, id=None, name=None, cover=None,
+            url=None):
         self.parentid = parent_id
         self.id = id
         self.name = name
         self.cover = cover
-        self.location = location
+        if( len(urlbase) and urlbase[-1] != '/'):
+            urlbase += '/'
+        self.url = urlbase + str(self.id)
+        self.location = AppleTrailerProxy(url)
         self.item = DIDLLite.VideoItem(id, parent_id, self.name)
+        self.item.albumArtURI = self.cover
+
+    def get_path(self):
+        return self.url
+
 
 class Container(BackendItem):
 
@@ -88,11 +104,7 @@ class AppleTrailersStore(BackendStore):
 
         dfr = self.update_data()
         # first get the first bunch of data before sending init_completed
-        dfr.addCallback(self._send_init)
-
-    def _send_init(self, result):
-        louie.send('Coherence.UPnP.Backend.init_completed',
-                None, backend=self)
+        dfr.addCallback(lambda x: self.init_completed())
 
     def queue_update(self, result):
         reactor.callLater(self.refresh, self.update_data)
@@ -128,9 +140,9 @@ class AppleTrailersStore(BackendStore):
         data['id'] = item.get('id')
         data['name'] = item.find('./info/title').text
         data['cover'] = item.find('./poster/location').text
-        data['location'] = item.find('./preview/large').text
+        data['url'] = item.find('./preview/large').text
 
-        trailer = Trailer(ROOT_ID, **data)
+        trailer = Trailer(ROOT_ID, self.urlbase, **data)
         duration = None
         try:
             hours = 0
@@ -158,7 +170,7 @@ class AppleTrailersStore(BackendStore):
         except:
             pass
 
-        res = DIDLLite.Resource(trailer.location, 'http-get:*:video/quicktime:*')
+        res = DIDLLite.Resource(trailer.get_path(), 'http-get:*:video/quicktime:*')
         res.duration = duration
         try:
             res.size = item.find('./preview/large').get('filesize',None)
@@ -194,10 +206,13 @@ class AppleTrailersStore(BackendStore):
         self.trailers[trailer.id] = trailer
 
     def get_by_id(self, id):
-        if int(id) == 0:
-            return self.container
-        else:
-            return self.trailers.get(id,None)
+        try:
+            if int(id) == 0:
+                return self.container
+            else:
+                return self.trailers.get(id,None)
+        except:
+            return None
 
     def upnp_init(self):
         if self.server:
@@ -207,3 +222,6 @@ class AppleTrailersStore(BackendStore):
         trailers = self.trailers.values()
         trailers.sort(cmp=lambda x,y : cmp(x.get_name().lower(),y.get_name().lower()))
         self.container.children = trailers
+
+    def __repr__(self):
+        return self.__class__.__name__
\ No newline at end of file
