@@ -10,6 +10,7 @@ import traceback
 import copy
 
 from twisted.python import filepath, util
+from twisted.internet.tcp import CannotListenError
 from twisted.internet import task, address, defer
 from twisted.internet import reactor
 from twisted.web import resource,static
@@ -41,38 +42,47 @@ class SimpleRoot(resource.Resource, log.Loggable):
     def __init__(self, coherence):
         resource.Resource.__init__(self)
         self.coherence = coherence
+        self.http_hostname = '%s:%d' % (self.coherence.hostname, self.coherence.web_server_port)
 
     def getChild(self, name, request):
         self.debug('SimpleRoot getChild %s, %s' % (name, request))
         if name == 'oob':
             """ we have an out-of-band request """
             return static.File(self.coherence.dbus.pinboard[request.args['key'][0]])
+
+        if name == '':
+            return self
+
+        # at this stage, name should be a device UUID
         try:
             return self.coherence.children[name]
         except:
-            return self
+            self.warning("Cannot find device for requested name:", name)
+            request.setResponseCode(404)
+            return static.Data('<html><p>No device for requested UUID: %s</p></html>' % name,'text/html')
+
 
     def listchilds(self, uri):
         self.info('listchilds %s' % uri)
         if uri[-1] != '/':
             uri += '/'
-        cl = ''
-        for c in self.coherence.children:
-            device = self.coherence.get_device_with_id(c)
-            if device != None:
-                _,_,_,device_type,version = device.get_device_type().split(':')
-                cl +=  '<li><a href=%s%s>%s:%s %s</a></li>' % (
-                                        uri,c,
-                                        device_type.encode('utf-8'), version.encode('utf-8'),
-                                        device.get_friendly_name().encode('utf-8'))
+        cl = []
+        for child in self.coherence.children:
+            device = self.coherence.get_device_with_id(child)
+            if device is not None:
+                cl.append('<li><a href=%s%s>%s:%s %s</a></li>' % (
+                        uri, child, device.get_friendly_device_type(),
+                        device.get_device_type_version(),
+                        device.get_friendly_name()))
 
-        for c in self.children:
-                cl += '<li><a href=%s%s>%s</a></li>' % (uri,c,c)
-        return cl
+        for child in self.children:
+                cl.append('<li><a href=%s%s>%s</a></li>' % (uri, child, child))
+        return "".join(cl)
 
     def render(self,request):
-        return """<html><head><title>Coherence</title></head><body>
+        result = """<html><head><title>Coherence</title></head><body>
 <a href="http://coherence.beebits.net">Coherence</a> - a Python DLNA/UPnP framework for the Digital Living<p>Hosting:<ul>%s</ul></p></body></html>""" % self.listchilds(request.uri)
+        return result.encode('utf-8')
 
 
 class WebServer(log.Loggable):
@@ -85,6 +95,10 @@ class WebServer(log.Loggable):
                     the web ui """
                 raise ImportError
 
+            self.warning("Web UI not supported atm, will return with version 0.7.0")
+            raise ImportError
+
+
             from nevow import __version_info__, __version__
             if __version_info__ <(0,9,17):
                 self.warning( "Nevow version %s too old, disabling WebUI" % __version__)
@@ -150,7 +164,7 @@ class Plugins(log.Loggable):
         self._plugins.__setitem__(key,value)
 
     def set(self, key,value):
-        return self.__getitem__(key,value)
+        return self.__setitem__(key,value)
 
     def keys(self):
         return self._plugins.keys()
@@ -165,7 +179,7 @@ class Plugins(log.Loggable):
                     try:
                         #print entrypoint, type(entrypoint)
                         self._plugins[entrypoint.name] = entrypoint.load(require=False)
-                    except (ImportError, pkg_resources.ResolutionError), msg:
+                    except (ImportError, AttributeError, pkg_resources.ResolutionError), msg:
                         self.warning("Can't load plugin %s (%s), maybe missing dependencies..." % (entrypoint.name,msg))
                         self.info(traceback.format_exc())
         except ImportError:
@@ -212,6 +226,7 @@ class Coherence(log.Loggable):
         self.cls._instance_ = None
 
     def setup(self, config={}):
+        self._mirabeau = None
 
         self.devices = []
         self.children = {}
@@ -259,40 +274,14 @@ class Coherence(log.Loggable):
             _debug = '*:%d' % log.human2level(logmode)
         try:
             logfile = config.get('logging').get('logfile',None)
+            if logfile != None:
+                logfile = unicode(logfile)
         except (KeyError,AttributeError,TypeError):
             logfile = config.get('logfile', None)
         log.init(logfile, _debug)
 
-        self.louieplugin = louie.TwistedDispatchPlugin()
-        try:
-            louie.install_plugin(self.louieplugin)
-        except louie.error.PluginTypeError:
-            """ we do need this to survive multiple calls
-                to Coherence during trial tests
-            """
-            pass
-
         self.warning("Coherence UPnP framework version %s starting..." % __version__)
 
-        unittest = config.get('unittest', 'no')
-        if unittest == 'no':
-            unittest = False
-        else:
-            unittest = True
-
-        self.ssdp_server = SSDPServer(test=unittest)
-        louie.connect( self.create_device, 'Coherence.UPnP.SSDP.new_device', louie.Any)
-        louie.connect( self.remove_device, 'Coherence.UPnP.SSDP.removed_device', louie.Any)
-        louie.connect( self.add_device, 'Coherence.UPnP.RootDevice.detection_completed', louie.Any)
-        #louie.connect( self.receiver, 'Coherence.UPnP.Service.detection_completed', louie.Any)
-
-        self.ssdp_server.subscribe("new_device", self.add_device)
-        self.ssdp_server.subscribe("removed_device", self.remove_device)
-
-        self.msearch = MSearch(self.ssdp_server,test=unittest)
-
-        reactor.addSystemEventTrigger( 'before', 'shutdown', self.shutdown, force=True)
-
         if network_if:
             self.hostname = get_ip_address('%s' % network_if)
         else:
@@ -304,12 +293,45 @@ class Coherence(log.Loggable):
 
         if self.hostname.startswith('127.'):
             """ use interface detection via routing table as last resort """
-            self.hostname = get_host_address()
+            def catch_result(hostname):
+                self.hostname = hostname
+                self.setup_part2()
+            d = defer.maybeDeferred(get_host_address)
+            d.addCallback(catch_result)
+        else:
+            self.setup_part2()
+
+    def setup_part2(self):
 
         self.info('running on host: %s' % self.hostname)
         if self.hostname.startswith('127.'):
             self.warning('detection of own ip failed, using %s as own address, functionality will be limited', self.hostname)
-        self.web_server = WebServer( config.get('web-ui',None), self.web_server_port, self)
+
+        unittest = self.config.get('unittest', 'no')
+        if unittest == 'no':
+            unittest = False
+        else:
+            unittest = True
+
+        self.ssdp_server = SSDPServer(test=unittest,interface=self.hostname)
+        louie.connect( self.create_device, 'Coherence.UPnP.SSDP.new_device', louie.Any)
+        louie.connect( self.remove_device, 'Coherence.UPnP.SSDP.removed_device', louie.Any)
+        louie.connect( self.add_device, 'Coherence.UPnP.RootDevice.detection_completed', louie.Any)
+        #louie.connect( self.receiver, 'Coherence.UPnP.Service.detection_completed', louie.Any)
+
+        self.ssdp_server.subscribe("new_device", self.add_device)
+        self.ssdp_server.subscribe("removed_device", self.remove_device)
+
+        self.msearch = MSearch(self.ssdp_server,test=unittest)
+
+        reactor.addSystemEventTrigger( 'before', 'shutdown', self.shutdown, force=True)
+
+        try:
+            self.web_server = WebServer( self.config.get('web-ui',None), self.web_server_port, self)
+        except CannotListenError:
+            self.warning('port %r already in use, aborting!' % self.web_server_port)
+            reactor.stop()
+            return
 
         self.urlbase = 'http://%s:%d/' % (self.hostname, self.web_server_port)
 
@@ -318,14 +340,16 @@ class Coherence(log.Loggable):
 
         self.available_plugins = None
 
+        self.ctrl = None
+
         try:
-            plugins = config['plugin']
+            plugins = self.config['plugin']
             if isinstance(plugins,dict):
                 plugins=[plugins]
         except:
             plugins = None
         if plugins is None:
-            plugins = config.get('plugins',None)
+            plugins = self.config.get('plugins',None)
 
         if plugins is None:
             self.info("No plugin defined!")
@@ -359,16 +383,47 @@ class Coherence(log.Loggable):
                         self.warning("Can't enable plugin, %s: %s!" % (plugin, msg))
                         self.info(traceback.format_exc())
 
-        if config.get('controlpoint', 'no') == 'yes':
+
+        self.external_address = ':'.join((self.hostname,str(self.web_server_port)))
+
+
+        if(self.config.get('controlpoint', 'no') == 'yes' or
+           self.config.get('json','no') == 'yes'):
             self.ctrl = ControlPoint(self)
 
-        if config.get('use_dbus', 'no') == 'yes':
+        if self.config.get('json','no') == 'yes':
+            from coherence.json import JsonInterface
+            self.json = JsonInterface(self.ctrl)
+
+        if self.config.get('transcoding', 'no') == 'yes':
+            from coherence.transcoder import TranscoderManager
+            self.transcoder_manager = TranscoderManager(self)
+
+
+        if self.config.get('use_dbus', 'no') == 'yes':
             try:
                 from coherence import dbus_service
+                if self.ctrl == None:
+                    self.ctrl = ControlPoint(self)
                 self.dbus = dbus_service.DBusPontoon(self.ctrl)
             except Exception, msg:
                 self.warning("Unable to activate dbus sub-system: %r" % msg)
                 self.debug(traceback.format_exc())
+            else:
+                if self.config.get('enable_mirabeau', 'no') == 'yes':
+                    from coherence import mirabeau
+                    from coherence.tube_service import MirabeauProxy
+
+                    mirabeau_cfg = self.config.get('mirabeau', {})
+
+                    try:
+                        self.external_address = mirabeau_cfg['external_address']
+                    except KeyError:
+                        pass
+
+                    self._mirabeau = mirabeau.Mirabeau(mirabeau_cfg, self)
+                    self.add_web_resource('mirabeau', MirabeauProxy())
+                    self._mirabeau.start()
 
     def add_plugin(self, plugin, **kwargs):
         self.info("adding plugin %r", plugin)
@@ -393,7 +448,7 @@ class Coherence(log.Loggable):
                 except Exception, msg:
                     self.warning("Can't enable %s plugin for sub-system %s, %s!" % (plugin, device, msg))
                     self.debug(traceback.format_exc())
-        except KeyError:
+        except KeyError, error:
             self.warning("Can't enable %s plugin, not found!" % plugin)
         except Exception, msg:
             self.warning("Can't enable %s plugin, %s!" % (plugin, msg))
@@ -466,9 +521,9 @@ class Coherence(log.Loggable):
         if self._incarnations_ > 1:
             self._incarnations_ -= 1
             return
-        if self.louieplugin != None:
-            louie.remove_plugin(self.louieplugin)
-            self.louieplugin = None
+        if self._mirabeau is not None:
+            self._mirabeau.stop()
+            self._mirabeau = None
         for backend in self.active_backends.itervalues():
             backend.unregister()
         self.active_backends = {}
@@ -575,17 +630,17 @@ class Coherence(log.Loggable):
         #    self.callback("new_device", infos['ST'], infos)
 
     def add_device(self, device):
-        self.info("adding device",device.get_usn())
+        self.info("adding device",device.get_id(),device.get_usn(),device.friendly_device_type)
         self.devices.append(device)
 
     def remove_device(self, device_type, infos):
         self.info("removed device",infos['ST'],infos['USN'])
         device = self.get_device_with_usn(infos['USN'])
         if device:
+            louie.send('Coherence.UPnP.Device.removed', None, usn=infos['USN'])
             self.devices.remove(device)
             device.remove()
             if infos['ST'] == 'upnp:rootdevice':
-                louie.send('Coherence.UPnP.Device.removed', None, usn=infos['USN'])
                 louie.send('Coherence.UPnP.RootDevice.removed', None, usn=infos['USN'])
                 self.callback("removed_device", infos['ST'], infos['USN'])
 
