@@ -15,6 +15,8 @@ from coherence import __version__
 
 from coherence.extern.et import ET, indent
 
+from coherence.upnp.core.utils import StaticFile
+
 from coherence.upnp.services.servers.connection_manager_server import ConnectionManagerServer
 from coherence.upnp.services.servers.rendering_control_server import RenderingControlServer
 from coherence.upnp.services.servers.av_transport_server import AVTransportServer
@@ -32,6 +34,7 @@ class MediaRenderer(log.Loggable,BasicDeviceMixin):
     device_type = 'MediaRenderer'
 
     def fire(self,backend,**kwargs):
+
         if kwargs.get('no_thread_needed',False) == False:
             """ this could take some time, put it in a  thread to be sure it doesn't block
                 as we can't tell for sure that every backend is implemented properly """
@@ -89,6 +92,11 @@ class MediaRenderer(log.Loggable,BasicDeviceMixin):
         self.web_resource = HttpRoot(self)
         self.coherence.add_web_resource( str(self.uuid)[5:], self.web_resource)
 
+        try:
+            dlna_caps = self.backend.dlna_caps
+        except AttributeError:
+            dlna_caps = []
+
 
         version = self.version
         while version > 0:
@@ -96,14 +104,16 @@ class MediaRenderer(log.Loggable,BasicDeviceMixin):
                                     RootDeviceXML( self.coherence.hostname,
                                     str(self.uuid),
                                     self.coherence.urlbase,
-                                    self.device_type, version,
+                                    device_type=self.device_type,
+                                    version=version,
                                     #presentation_url='/'+str(self.uuid)[5:],
                                     friendly_name=self.backend.name,
                                     model_description='Coherence UPnP A/V %s' % self.device_type,
                                     model_name='Coherence UPnP A/V %s' % self.device_type,
                                     services=self._services,
                                     devices=self._devices,
-                                    icons=self.icons))
+                                    icons=self.icons,
+                                    dlna_caps=dlna_caps))
             version -= 1
 
 
@@ -114,8 +124,19 @@ class MediaRenderer(log.Loggable,BasicDeviceMixin):
         for icon in self.icons:
             if icon.has_key('url'):
                 if icon['url'].startswith('file://'):
-                    self.web_resource.putChild(os.path.basename(icon['url']),
-                                               static.File(icon['url'][7:]))
+                    if os.path.exists(icon['url'][7:]):
+                        self.web_resource.putChild(os.path.basename(icon['url']),
+                                                   StaticFile(icon['url'][7:],defaultType=icon['mimetype']))
+                elif icon['url'] == '.face':
+                    face_path = os.path.abspath(os.path.join(os.path.expanduser('~'), ".face"))
+                    if os.path.exists(face_path):
+                        self.web_resource.putChild('face-icon.png',StaticFile(face_path,defaultType=icon['mimetype']))
+                else:
+                    from pkg_resources import resource_filename
+                    icon_path = os.path.abspath(resource_filename(__name__, os.path.join('..','..','..','misc','device-icons',icon['url'])))
+                    if os.path.exists(icon_path):
+                        self.web_resource.putChild(icon['url'],StaticFile(icon_path,defaultType=icon['mimetype']))
+
 
         self.register()
         self.warning("%s %s (%s) activated with %s" % (self.backend.name, self.device_type, self.backend, str(self.uuid)[5:]))
