@@ -29,6 +29,7 @@ from coherence.upnp.core.utils import ReverseProxyResource
 
 from coherence.upnp.services.servers.connection_manager_server import ConnectionManagerServer
 from coherence.upnp.services.servers.content_directory_server import ContentDirectoryServer
+from coherence.upnp.services.servers.scheduled_recording_server import ScheduledRecordingServer
 from coherence.upnp.services.servers.media_receiver_registrar_server import MediaReceiverRegistrarServer
 from coherence.upnp.services.servers.media_receiver_registrar_server import FakeMediaReceiverRegistrarBackend
 
@@ -68,6 +69,30 @@ class MSRoot(resource.Resource, log.Loggable):
         headers = request.getAllHeaders()
         self.msg( request.getAllHeaders())
 
+        try:
+            if headers['getcontentfeatures.dlna.org'] != '1':
+                request.setResponseCode(400)
+                return static.Data('<html><p>wrong value for getcontentFeatures.dlna.org</p></html>','text/html')
+        except:
+            pass
+
+        if request.method == 'HEAD':
+            if 'getcaptioninfo.sec' in headers:
+                self.warning("requesting srt file for id %s" % path)
+                ch = self.store.get_by_id(path)
+                try:
+                    location = ch.get_path()
+                    caption = ch.caption
+                    if caption == None:
+                        raise KeyError
+                    request.setResponseCode(200)
+                    request.setHeader('CaptionInfo.sec', caption)
+                    return static.Data('','text/html')
+                except:
+                    print traceback.format_exc()
+                    request.setResponseCode(404)
+                    return static.Data('<html><p>the requested srt file was not found</p></html>','text/html')
+
         try:
             request._dlna_transfermode = headers['transfermode.dlna.org']
         except KeyError:
@@ -75,43 +100,49 @@ class MSRoot(resource.Resource, log.Loggable):
         if request.method in ('GET','HEAD'):
             if COVER_REQUEST_INDICATOR.match(request.uri):
                 self.info("request cover for id %s" % path)
-                ch = self.store.get_by_id(path)
-                if ch is not None:
-                    request.setResponseCode(200)
-                    file = ch.get_cover()
-                    if os.path.exists(file):
-                        self.info("got cover %s" % file)
-                        return StaticFile(file)
-                request.setResponseCode(404)
-                return static.Data('<html><p>cover requested not found</p></html>','text/html')
+                def got_item(ch):
+                    if ch is not None:
+                        request.setResponseCode(200)
+                        file = ch.get_cover()
+                        if os.path.exists(file):
+                            self.info("got cover %s" % file)
+                            return StaticFile(file)
+                    request.setResponseCode(404)
+                    return static.Data('<html><p>cover requested not found</p></html>','text/html')
+
+                dfr = defer.maybeDeferred(self.store.get_by_id, path)
+                dfr.addCallback(got_item)
+                return dfr
 
             if ATTACHMENT_REQUEST_INDICATOR.match(request.uri):
                 self.info("request attachment %r for id %s" % (request.args,path))
-                ch = self.store.get_by_id(path)
-                try:
-                    #FIXME same as below
-                    if 'transcoded' in request.args:
-                        if self.server.coherence.config.get('transcoding', 'no') == 'yes':
-                            format = request.args['transcoded'][0]
-                            type = request.args['type'][0]
-                            self.info("request transcoding %r %r" % (format, type))
-                            if format == 'thumb' and type == 'jpeg':
+                def got_attachment(ch):
+                    try:
+                        #FIXME same as below
+                        if 'transcoded' in request.args:
+                            if self.server.coherence.config.get('transcoding', 'no') == 'yes':
+                                format = request.args['transcoded'][0]
+                                type = request.args['type'][0]
+                                self.info("request transcoding %r %r" % (format, type))
                                 try:
-                                    from coherence.transcoder import JPEGThumbTranscoder
-                                    return JPEGThumbTranscoder(ch.item.attachments[request.args['attachment'][0]])
+                                    from coherence.transcoder import TranscoderManager
+                                    manager = TranscoderManager(self.server.coherence)
+                                    return manager.select(format,ch.item.attachments[request.args['attachment'][0]])
                                 except:
                                     self.debug(traceback.format_exc())
-                            request.setResponseCode(404)
-                            return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+                                request.setResponseCode(404)
+                                return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+                            else:
+                                request.setResponseCode(404)
+                                return static.Data("<html><p>This MediaServer doesn't support transcoding</p></html>",'text/html')
                         else:
-                            request.setResponseCode(404)
-                            return static.Data("<html><p>This MediaServer doesn't support transcoding</p></html>",'text/html')
-                    else:
-                        return ch.item.attachments[request.args['attachment'][0]]
-                except:
-                    request.setResponseCode(404)
-                    return static.Data('<html><p>the requested attachment was not found</p></html>','text/html')
-
+                            return ch.item.attachments[request.args['attachment'][0]]
+                    except:
+                        request.setResponseCode(404)
+                        return static.Data('<html><p>the requested attachment was not found</p></html>','text/html')
+                dfr = defer.maybeDeferred(self.store.get_by_id, path)
+                dfr.addCallback(got_attachment)
+                return dfr
         #if(request.method in ('GET','HEAD') and
         #   XBOX_TRANSCODED_REQUEST_INDICATOR.match(request.uri)):
         #    if self.server.coherence.config.get('transcoding', 'no') == 'yes':
@@ -125,52 +156,22 @@ class MSRoot(resource.Resource, log.Loggable):
            TRANSCODED_REQUEST_INDICATOR.match(request.uri)):
             self.info("request transcoding to %s for id %s" % (request.uri.split('/')[-1],path))
             if self.server.coherence.config.get('transcoding', 'no') == 'yes':
-                ch = self.store.get_by_id(path)
-                #FIXME create a generic transcoder class and sort the details there
-                format = request.uri.split('/')[-1] #request.args['transcoded'][0]
-                uri = ch.get_path()
-                if format == 'lpcm':
+                def got_stuff_to_transcode(ch):
+                    #FIXME create a generic transcoder class and sort the details there
+                    format = request.uri.split('/')[-1] #request.args['transcoded'][0]
+                    uri = ch.get_path()
                     try:
-                        from coherence.transcoder import PCMTranscoder
-                        return PCMTranscoder(uri)
+                        from coherence.transcoder import TranscoderManager
+                        manager = TranscoderManager(self.server.coherence)
+                        return manager.select(format,uri)
                     except:
                         self.debug(traceback.format_exc())
                         request.setResponseCode(404)
                         return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
-                if format == 'wav':
-                    try:
-                        from coherence.transcoder import WAVTranscoder
-                        return WAVTranscoder(uri)
-                    except:
-                        self.debug(traceback.format_exc())
-                        request.setResponseCode(404)
-                        return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
-                if format == 'mp3':
-                    try:
-                        from coherence.transcoder import MP3Transcoder
-                        return MP3Transcoder(uri)
-                    except:
-                        self.debug(traceback.format_exc())
-                        request.setResponseCode(404)
-                        return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
-                if format == 'mp4':
-                    try:
-                        from coherence.transcoder import MP4Transcoder
-                        return MP4Transcoder(uri)
-                    except:
-                        self.debug(traceback.format_exc())
-                        request.setResponseCode(404)
-                        return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
-                if format == 'thumb':
-                    type = request.args['type'][0]
-                    if type == 'jpeg':
-                        try:
-                            from coherence.transcoder import JPEGThumbTranscoder
-                            return JPEGThumbTranscoder(uri)
-                        except:
-                            self.debug(traceback.format_exc())
-                            request.setResponseCode(404)
-                            return static.Data('<html><p>the requested transcoded file was not found</p></html>','text/html')
+                dfr = defer.maybeDeferred(self.store.get_by_id, path)
+                dfr.addCallback(got_stuff_to_transcode)
+                return dfr
+
             request.setResponseCode(404)
             return static.Data("<html><p>This MediaServer doesn't support transcoding</p></html>",'text/html')
 
@@ -206,17 +207,38 @@ class MSRoot(resource.Resource, log.Loggable):
     def import_file(self,name,request):
         self.info("import file, id %s" % name)
         print "import file, id %s" % name
-        ch = self.store.get_by_id(name)
-        print "ch", ch
-        if ch is not None:
-            if hasattr(self.store,'backend_import'):
-                response_code = self.store.backend_import(ch,request.content)
-                if isinstance(response_code, defer.Deferred):
-                    return response_code
-                request.setResponseCode(response_code)
-                return
+        def got_file(ch):
+            print "ch", ch
+            if ch is not None:
+                if hasattr(self.store,'backend_import'):
+                    response_code = self.store.backend_import(ch,request.content)
+                    if isinstance(response_code, defer.Deferred):
+                        return response_code
+                    request.setResponseCode(response_code)
+                    return
+            else:
+                request.setResponseCode(404)
+        dfr = defer.maybeDeferred(self.store.get_by_id, name)
+        dfr.addCallback(got_file)
 
-        request.setResponseCode(404)
+    def prepare_connection(self,request):
+        new_id,_,_ = self.server.connection_manager_server.add_connection('',
+                                                                    'Output',
+                                                                    -1,
+                                                                    '')
+        self.info("startup, add %d to connection table" % new_id)
+        d = request.notifyFinish()
+        d.addBoth(self.requestFinished, new_id, request)
+
+    def prepare_headers(self,ch,request):
+        request.setHeader('transferMode.dlna.org', request._dlna_transfermode)
+        if hasattr(ch,'item') and hasattr(ch.item, 'res'):
+            if ch.item.res[0].protocolInfo is not None:
+                additional_info = ch.item.res[0].get_additional_info()
+                if additional_info != '*':
+                    request.setHeader('contentFeatures.dlna.org', additional_info)
+                elif 'getcontentfeatures.dlna.org' in request.getAllHeaders():
+                    request.setHeader('contentFeatures.dlna.org', "DLNA.ORG_OP=01;DLNA.ORG_CI=0;DLNA.ORG_FLAGS=01500000000000000000000000000000")
 
     def process_child(self,ch,name,request):
         if ch != None:
@@ -240,21 +262,12 @@ class MSRoot(resource.Resource, log.Loggable):
                     request.content = StringIO()
 
             if hasattr(ch, "location"):
-                if isinstance(ch.location, ReverseProxyResource):
+                self.debug("we have a location %s" % isinstance(ch.location, resource.Resource))
+                if(isinstance(ch.location, ReverseProxyResource) or
+                   isinstance(ch.location, resource.Resource)):
                     self.info('getChild proxy %s to %s' % (name, ch.location.uri))
-                    new_id,_,_ = self.server.connection_manager_server.add_connection('',
-                                                                                'Output',
-                                                                                -1,
-                                                                                '')
-                    self.info("startup, add %d to connection table" % new_id)
-                    d = request.notifyFinish()
-                    d.addBoth(self.requestFinished, new_id, request)
-                    request.setHeader('transferMode.dlna.org', request._dlna_transfermode)
-                    if hasattr(ch,'item') and hasattr(ch.item, 'res'):
-                        if ch.item.res[0].protocolInfo is not None:
-                            additional_info = ch.item.res[0].get_additional_info()
-                            if additional_info != '*':
-                                request.setHeader('contentFeatures.dlna.org', additional_info)
+                    self.prepare_connection(request)
+                    self.prepare_headers(ch,request)
                     return ch.location
             try:
                 p = ch.get_path()
@@ -266,19 +279,8 @@ class MSRoot(resource.Resource, log.Loggable):
                 return self.list_content(name, ch, request)
             if p != None and os.path.exists(p):
                 self.info("accessing path %r" % p)
-                new_id,_,_ = self.server.connection_manager_server.add_connection('',
-                                                                            'Output',
-                                                                            -1,
-                                                                            '')
-                self.info("startup, add %d to connection table" % new_id)
-                d = request.notifyFinish()
-                d.addBoth(self.requestFinished, new_id, request)
-                request.setHeader('transferMode.dlna.org', request._dlna_transfermode)
-                if hasattr(ch, 'item') and hasattr(ch.item, 'res'):
-                    if ch.item.res[0].protocolInfo is not None:
-                        additional_info = ch.item.res[0].get_additional_info()
-                        if additional_info != '*':
-                            request.setHeader('contentFeatures.dlna.org', additional_info)
+                self.prepare_connection(request)
+                self.prepare_headers(ch,request)
                 ch = StaticFile(p)
             else:
                 self.debug("accessing path %r failed" % p)
@@ -395,7 +397,8 @@ class RootDeviceXML(static.Data):
                         xbox_hack=False,
                         services=[],
                         devices=[],
-                        icons=[]):
+                        icons=[],
+                        presentationURL=None):
         uuid = str(uuid)
         root = ET.Element('root')
         root.attrib['xmlns']='urn:schemas-upnp-org:device-1-0'
@@ -408,31 +411,52 @@ class RootDeviceXML(static.Data):
         #    ET.SubElement(root, 'URLBase').text = urlbase + uuid[5:] + '/'
 
         d = ET.SubElement(root, 'device')
-        x = ET.SubElement(d, 'dlna:X_DLNADOC')
-        x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
-        x.text = 'DMS-1.50'
-        x = ET.SubElement(d, 'dlna:X_DLNADOC')
-        x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
-        x.text = 'M-DMS-1.50'
-        x=ET.SubElement(d, 'dlna:X_DLNACAP')
-        x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
-        x.text = 'av-upload,image-upload,audio-upload'
         ET.SubElement(d, 'deviceType').text = device_type
         if xbox_hack == False:
-            ET.SubElement(d, 'modelName').text = 'Coherence UPnP A/V MediaServer'
             ET.SubElement(d, 'friendlyName').text = friendly_name
         else:
-            ET.SubElement(d, 'modelName').text = 'Windows Media Connect'
             ET.SubElement(d, 'friendlyName').text = friendly_name + ' : 1 : Windows Media Connect'
         ET.SubElement(d, 'manufacturer').text = 'beebits.net'
         ET.SubElement(d, 'manufacturerURL').text = 'http://coherence.beebits.net'
         ET.SubElement(d, 'modelDescription').text = 'Coherence UPnP A/V MediaServer'
+        if xbox_hack == False:
+            ET.SubElement(d, 'modelName').text = 'Coherence UPnP A/V MediaServer'
+        else:
+            ET.SubElement(d, 'modelName').text = 'Windows Media Connect'
         ET.SubElement(d, 'modelNumber').text = __version__
         ET.SubElement(d, 'modelURL').text = 'http://coherence.beebits.net'
         ET.SubElement(d, 'serialNumber').text = '0000001'
         ET.SubElement(d, 'UDN').text = uuid
         ET.SubElement(d, 'UPC').text = ''
-        ET.SubElement(d, 'presentationURL').text = '/' + uuid[5:]
+
+        if len(icons):
+            e = ET.SubElement(d, 'iconList')
+            for icon in icons:
+
+                icon_path = ''
+                if icon.has_key('url'):
+                    if icon['url'].startswith('file://'):
+                        icon_path = icon['url'][7:]
+                    elif icon['url'] == '.face':
+                        icon_path = os.path.join(os.path.expanduser('~'), ".face")
+                    else:
+                        from pkg_resources import resource_filename
+                        icon_path = os.path.abspath(resource_filename(__name__, os.path.join('..','..','..','misc','device-icons',icon['url'])))
+
+                if os.path.exists(icon_path) == True:
+                    i = ET.SubElement(e, 'icon')
+                    for k,v in icon.items():
+                        if k == 'url':
+                            if v.startswith('file://'):
+                                ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+os.path.basename(v)
+                                continue
+                            elif v == '.face':
+                                ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+'face-icon.png'
+                                continue
+                            else:
+                                ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+os.path.basename(v)
+                                continue
+                        ET.SubElement(i, k).text = str(v)
 
         if len(services):
             e = ET.SubElement(d, 'serviceList')
@@ -467,22 +491,19 @@ class RootDeviceXML(static.Data):
         if len(devices):
             e = ET.SubElement(d, 'deviceList')
 
-        if len(icons):
-            e = ET.SubElement(d, 'iconList')
-            for icon in icons:
-                i = ET.SubElement(e, 'icon')
-                for k,v in icon.items():
-                    if k == 'url':
-                        if v.startswith('file://'):
-                            ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+os.path.basename(v)
-                            continue
-                        elif v == '.face':
-                            ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+'face-icon.png'
-                            continue
-                        else:
-                            ET.SubElement(i, k).text = '/'+uuid[5:]+'/'+os.path.basename(v)
-                            continue
-                    ET.SubElement(i, k).text = str(v)
+        if presentationURL is None:
+            presentationURL = '/' + uuid[5:]
+        ET.SubElement(d, 'presentationURL').text = presentationURL
+
+        x = ET.SubElement(d, 'dlna:X_DLNADOC')
+        x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
+        x.text = 'DMS-1.50'
+        x = ET.SubElement(d, 'dlna:X_DLNADOC')
+        x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
+        x.text = 'M-DMS-1.50'
+        x=ET.SubElement(d, 'dlna:X_DLNACAP')
+        x.attrib['xmlns:dlna']='urn:schemas-dlna-org:device-1-0'
+        x.text = 'av-upload,image-upload,audio-upload'
 
         #if self.has_level(LOG_DEBUG):
         #    indent( root)
@@ -493,8 +514,11 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
     logCategory = 'mediaserver'
 
     device_type = 'MediaServer'
+    
+    presentationURL = None
 
     def fire(self,backend,**kwargs):
+
         if kwargs.get('no_thread_needed',False) == False:
             """ this could take some time, put it in a  thread to be sure it doesn't block
                 as we can't tell for sure that every backend is implemented properly """
@@ -547,6 +571,12 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
         except LookupError,msg:
             self.warning( 'MediaReceiverRegistrarServer (optional)', msg)
 
+        try:
+            self.scheduled_recording_server = ScheduledRecordingServer(self)
+            self._services.append(self.scheduled_recording_server)
+        except LookupError,msg:
+            self.info( 'ScheduledRecordingServer', msg)
+
         upnp_init = getattr(self.backend, "upnp_init", None)
         if upnp_init:
             upnp_init()
@@ -564,7 +594,8 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
                                     friendly_name=self.backend.name,
                                     services=self._services,
                                     devices=self._devices,
-                                    icons=self.icons))
+                                    icons=self.icons,
+                                    presentationURL = self.presentationURL))
             self.web_resource.putChild( 'xbox-description-%d.xml' % version,
                                     RootDeviceXML( self.coherence.hostname,
                                     str(self.uuid),
@@ -574,28 +605,32 @@ class MediaServer(log.Loggable,BasicDeviceMixin):
                                     xbox_hack=True,
                                     services=self._services,
                                     devices=self._devices,
-                                    icons=self.icons))
+                                    icons=self.icons,
+                                    presentationURL = self.presentationURL))
             version -= 1
 
         self.web_resource.putChild('ConnectionManager', self.connection_manager_server)
         self.web_resource.putChild('ContentDirectory', self.content_directory_server)
+        if hasattr(self,"scheduled_recording_server"):
+            self.web_resource.putChild('ScheduledRecording', self.scheduled_recording_server)
         if hasattr(self,"media_receiver_registrar_server"):
             self.web_resource.putChild('X_MS_MediaReceiverRegistrar', self.media_receiver_registrar_server)
 
         for icon in self.icons:
             if icon.has_key('url'):
                 if icon['url'].startswith('file://'):
-                    self.web_resource.putChild(os.path.basename(icon['url']),
-                                               StaticFile(icon['url'][7:]))
+                    if os.path.exists(icon['url'][7:]):
+                        self.web_resource.putChild(os.path.basename(icon['url']),
+                                                   StaticFile(icon['url'][7:],defaultType=icon['mimetype']))
                 elif icon['url'] == '.face':
-                    face_path = os.path.join(os.path.expanduser('~'), ".face")
+                    face_path = os.path.abspath(os.path.join(os.path.expanduser('~'), ".face"))
                     if os.path.exists(face_path):
-                        self.web_resource.putChild('face-icon.png',StaticFile(face_path))
+                        self.web_resource.putChild('face-icon.png',StaticFile(face_path,defaultType=icon['mimetype']))
                 else:
                     from pkg_resources import resource_filename
                     icon_path = os.path.abspath(resource_filename(__name__, os.path.join('..','..','..','misc','device-icons',icon['url'])))
                     if os.path.exists(icon_path):
-                        self.web_resource.putChild(icon['url'],StaticFile(icon_path))
+                        self.web_resource.putChild(icon['url'],StaticFile(icon_path,defaultType=icon['mimetype']))
 
         self.register()
         self.warning("%s %s (%s) activated with %s" % (self.backend.name, self.device_type, self.backend, str(self.uuid)[5:]))
